package routing;

import core.*;
import org.apache.commons.lang3.tuple.MutablePair;
import org.apache.commons.lang3.tuple.Pair;
import routing.util.SnifferRouter;

import java.util.HashMap;
import java.util.Map;

public class Router extends WatchdogRouter implements
        SnifferRouter {
    private Map<DTNHost, MutablePair<Double, Double>> neighbourRouting = new
            HashMap<>();
    public Router(Settings s) {
        super(s);
    }
    public Router(Router r) {
        super(r);
    }
    public Router replicate() {
        return new Router(this);
    }
    @Override
    protected int checkReceiving(Message m, DTNHost from) {
        int status = super.checkReceiving(m, from);
        if (status == TRY_LATER_BUSY) {
            return status;
        }

        DTNHost host = this.getHost();
        for (Connection connection : host.getConnections()) {
            Pair<NetworkInterface, NetworkInterface> ifaces = connection
                    .getInterfaces();
            NetworkInterface fromIface = ifaces.getLeft();
            NetworkInterface toIface = ifaces.getRight();
            DTNHost toHost = toIface.getHost();
            MessageRouter toHostRouter = toHost.getRouter();
            if (toHostRouter instanceof SnifferRouter) {
                ((SnifferRouter) toHostRouter).sniffReceive(m, host);
            }
        }
        return status;
    }
    @Override
    protected int startTransfer(Message m, Connection con) {
        int status = super.startTransfer(m, con);
        if (status == TRY_LATER_BUSY) {
            return status;
        }
        DTNHost host = this.getHost();
        for (Connection connection : host.getConnections()) {
            Pair<NetworkInterface, NetworkInterface> ifaces = connection
                    .getInterfaces();
            NetworkInterface fromIface = ifaces.getLeft();
            NetworkInterface toIface = ifaces.getRight();
            DTNHost toHost = toIface.getHost();
            MessageRouter toHostRouter = toHost.getRouter();
            if (toHostRouter instanceof SnifferRouter) {
                ((SnifferRouter) toHostRouter).sniffSend(m, host);
            }
        }
        return status;
    }

    @Override
    public void sniffReceive(Message m, DTNHost receiver) {
		super.sniffReceive();
        if (m != null && m.getTo() == receiver) {
            return;
        }

        if (this.neighbourRouting.containsKey(receiver)) {
            this.neighbourRouting.get(receiver).right++;
        } else {
            this.neighbourRouting.put(receiver, new MutablePair<>(1.0, 2.0));
        }
    }

    @Override
    public void sniffSend(Message m, DTNHost sender) {
		super.sniffSend();
        if (m != null && m.getFrom() == sender) {
            return;
        }
        if (this.neighbourRouting.containsKey(sender)) {
            this.neighbourRouting.get(sender).left++;
        } else {
            this.neighbourRouting.put(sender, new MutablePair<>(2.0, 1.0));
        }
    }

    @Override
    public void update() {
        super.update();
    }
}
